package physbeans.editors;

import java.awt.*;
import java.beans.*;
import javax.swing.*;
import physbeans.inout.*;
import physbeans.model.*;

/**
 * The panel for the BoundedRealModel editor
 */
public class BoundedRealModelCustomEditor extends JPanel {
  
  protected NumericTextLine minText, valText, maxText;
  protected BoundedRealModelEditor editor;
  
  protected final int cols = 16;   // size of text fields
  protected final int prec = 12;   // precision of text fields
  
  /**
   * Creates new BoundedRealModelCustomEditor.
   */
  public BoundedRealModelCustomEditor(BoundedRealModelEditor ed) {
    editor = ed;
    BoundedRealModel model = (BoundedRealModel) editor.getValue();
    if (model == null) {
      model = new BoundedRealModel();
    }
    
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    int offset = 30;
    minText = new NumericTextLine(model.getMinimum(), cols, prec, 
        "Minimum = ", "", "", offset);
    minText.addPropertyChangeListener(new MyTextListener());
    add(minText);
    
    valText = new NumericTextLine(model.getValue(), cols, prec, 
        "Value = ", "", "", offset);
    valText.addPropertyChangeListener(new MyTextListener());
    add(valText);
    
    maxText = new NumericTextLine(model.getMaximum(), cols, prec, 
        "Maximum = ", "", "", offset);
    maxText.addPropertyChangeListener(new MyTextListener());
    add(maxText);
    setPreferredSize(new Dimension(280, 160));
  }
  
  /**
   * Called when the value of a textfield is changed.
   */
  protected void changeValue() {
    BoundedRealModel newValue = getValueFromTextFields();
    
    editor.setValue(newValue);
    editor.firePropertyChange();
  }
  
  /**
   * Gets BoundedRealModel from the text fields.
   */
  protected BoundedRealModel getValueFromTextFields() {
    double min = minText.getValue();
    double val = valText.getValue();
    double max = maxText.getValue();
    return new BoundedRealModel(min, val, max);
  }
  
  /**
   * Sets text fields from BoundedRealModel.
   */
  protected void setTextFieldsToVector(BoundedRealModel newVal) {
    minText.setValue(newVal.getMinimum());
    valText.setValue(newVal.getValue());
    maxText.setValue(newVal.getMaximum());
  }
  
  /**
   * Listener to change in the text fields.
   */
  public class MyTextListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      changeValue();
    }
  }
}

