package physbeans.editors;

import java.awt.geom.*;
import java.awt.font.*;
import java.awt.*;
import java.beans.*;
import physbeans.math.*;

/**
 * Property editor for IVector.
 */
public class IVectorEditor extends PropertyEditorSupport {

  /**
   * Returns a string that represents the current DVector.
   */
  public String getAsText() {
    IVector v = (IVector) getValue();

    if (v == null) {
      return "null";
    } else {
      return v.toString();
    }
  }

  /**
   * Reads the value of the property by parsing a string.
   */
  public void setAsText(String text) throws IllegalArgumentException {
    if (text.equals("null") || text.equals("")) {
      setValue(null);
      return;
    }

    setValue(IVector.valueOf(text));
  }

  /**
   * Returns a string with java code that initializes a DVector.
   */
  public String getJavaInitializationString() {
    IVector v = (IVector) getValue();
    return "IVector.valueOf(\"" + v.toString() + "\")";
  }

  /**
   * Returns true if a custom editor exists.
   */
  public boolean supportsCustomEditor() {
    return true;
  }

  /**
   * Returns an instance of a new custom editor.
   */
  public Component getCustomEditor() {
    return new IVectorCustomEditor(this);
  }
}
