package physbeans.event;

import java.util.*;

/**
 * class to simplify use of VectorEvents
 */
public class VectorSupport {
  
  /** list of all observer objects listening  */
  protected Vector<VectorListener> listeners;
  
  /** Creates a new instance of VectorSupport */
  public VectorSupport() {
    // prepare for listeners
    listeners = new Vector<VectorListener>();
  }
  
  /**
   * register a VectorListener
   */
  public synchronized void addVectorListener(VectorListener sl) {
    listeners.addElement(sl);
  }
  
  /**
   * de-register a VectorListener
   */
  public synchronized void removeVectorListener(VectorListener sl) {
    listeners.removeElement(sl);
  }
  
  /**
   * inform all VectorListeners that the value of the vector has changed
   */
  public void fireVectorEvent(VectorEvent e) {
    Vector l;
    
    // clone the list of listeners so that delivery and simultaneous
    // addition/deletion of listeners don't interfere
    synchronized (this) {
      l = (Vector) listeners.clone();
    }
    
    for (int i = 0; i < l.size(); i++) {
      ((VectorListener) l.elementAt(i)).valueChanged(e);
    }
  }
}
