package physbeans.func;

/**
 * suppresses input until a given delay time is reached
 */
public class DelayedOn extends OneParameterFunction {
  
  protected double delayTime;
  protected double nullValue;
  protected double time;
  
  /**
   * creates default DelayedOn
   */
  public DelayedOn() {
    super(-1);
    delayTime = 10.0;
    time = 0.0;
    nullValue = 0.0;
  }
  
  /** returns input value or null value according to current time */
  public double evaluate(double d) {
    double result;
    if (time < delayTime) {
      result = nullValue;
    } else {
      result = d;
    }
    
    return result;
  }
  
  /**
   * gets the delay time
   */
  public double getDelayTime() {
    return this.delayTime;
  }
  
  /**
   * sets the delay time
   */
  public void setDelayTime(double delayTime) {
    this.delayTime = delayTime;
  }
  
  /**
   * gets the value for the output signals before delay time is reached
   */
  public double getNullValue() {
    return this.nullValue;
  }
  
  /**
   * sets the value for the output signals before delay time is reached
   */
  public void setNullValue(double nullValue) {
    this.nullValue = nullValue;
  }
  
  /**
   * gets the curren time
   */
  public double getTime() {
    return this.time;
  }
  
  /**
   * sets the curren time
   */
  public void setTime(double time) {
    this.time = time;
  }
  
}
