package physbeans.func;

import java.util.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * Base class of random vector functions.
 */
abstract public class RandomVector extends GenericVectorFunction
  implements Real1DFunction {
  
  protected boolean randomize;
  protected Random rng;
  protected int seed;
  
  /** Creates a new instance of RandomVector */
  public RandomVector() {
    super(0,1,100);
    seed = 0;
    randomize = false;
    rng = new Random(seed);
  }
  
  /**
   * Gets seed to start the random generator.
   */
  public int getSeed() {
    return this.seed;
  }  
  
  /**
   * Gets randomize flag.
   * If true, current time is used for initialization of random generator.
   */
  public boolean isRandomize() {
    return this.randomize;
  }
  
    /**
   * Sets size of output vector.
   */
  public void setOutputSize(int n) {
    super.setOutputSize(n);
    if (autoTriggered) {
      trigger();
    }
  }
    
  /**
   * Sets randomize flag.
   * If true, current time is used for initialization of random generator.
   * In this case, the seed is ignored.
   */
  public void setRandomize(boolean randomize) {
    this.randomize = randomize;
    rng = new Random();
  } 
  
  /**
   * Sets seed to start the random generator.
   */
  public void setSeed(int seed) {
    this.seed = seed;
    rng = new Random(seed);
  }

   /**
   * Creates new random numbers.
   */
  protected void compute() {
    int n = getOutputSize();
    for (int i = 0; i < n; i++) {
      outputValues[0].set(i, evaluate(0.0));
    }
  }

  /**
   * Computes the next random value.
   * The input argument is ignored.
   */
  abstract public double evaluate(double t);


 /**
   * Gets first element of input vector.<br>
   * Conveniance function for the scalar case.
   */
  public double getOutputValue() {
    return getOutputVector().get(0);
  }

  /**
   * Sets input vector with one element.<br>
   * Conveniance function for the scalar case.
   */
  public void setInputValue(double t) {
    trigger();
  }

}
