package physbeans.func;

/**
 * Combines a set of states.
 * <P>Each state is initially inactive and can be set to active.
 * When all states are active the combiner state is true else false.
 */
public class StateCombiner {
  
  protected boolean[] states;
  
  /**
   * Creates a default StateCombiner with two states.
   */
  public StateCombiner() {
    this(2);
  }
  
  /**
   * Creates a StateCombiner with n states.
   */
  public StateCombiner(int n) {
    states = new boolean[n];
  }
  
  /**
   * Sets the state i to active.
   */
  public void setState(int i) {
    states[i] = true;
  }
  
  /**
   * Sets all states to inactive.
   */
  public void reset() {
    for (int i = 0; i < states.length; i++) {
      states[i] = false;
    }
  }
  
  /**
   * Returns the current state of the combiner.
   * <p>It is true, when all inputs are active.
   */
  public boolean getCombinedState() {
    boolean ok = true;
    for (int i = 0; i < states.length; i++) {
      ok = ok && states[i];
    }
    return ok;
  }
}
