package physbeans.inout;

import physbeans.model.*;
import physbeans.views.*;


/**
 * Oscilloscope displaying several input values as functions of time.<br>
 * When a set of new values arrives, the internal time is increased by a fixed
 * amount.
 */
public class Oscilloscope extends XYScope {
  
  protected boolean automaticBufferSize;
  
  /**
   * Creates a new instance of Oscilloscope
   */
  public Oscilloscope() {
    super();
  }
  
  protected void createGUI() {
    setScaleX(false);
    setScaleY(true);
    
    data = new TimedDataBuffer();
    setBufferSize(10);
    setNLines(2);
    
    dataView = new TimedDataBufferView();
    dataView.setModel(data);
    screen.addView(dataView);
    
    automaticBufferSize = true;
  }
  
  /**
   * Gets the time per tick, i.e. the amount by which the internal time is
   * incremented each time a set of new values arrives.
   */
  public double getTimePerTick() {
    return ((TimedDataBuffer) data).getTimePerTick();
  }
  
  /**
   * Sets the time per tick, i.e. the amount by which the internal time is
   * incremented each time a set of new values arrives.
   */
  public void setTimePerTick(double dt) {
    ((TimedDataBuffer) data).setTimePerTick(dt);
  }
  
  /**
   * Gets the value of the automaticBufferSize flag.<br>
   * If true, the buffer size is adapted to the screen size. Otherwise the
   * explicitly given buffer size is used.
   */
  public boolean isAutomaticBufferSize() {
    return automaticBufferSize;
  }
  
  /**
   * Sets the value of the automaticBufferSize flag.<br>
   * If true, the buffer size is adapted to the screen size. Otherwise the
   * explicitly given buffer size is used.
   */
  public void setAutomaticBufferSize(boolean b) {
    automaticBufferSize = b;
  }
  
  /**
   * Adapts the buffer size to the screen size, if necessary.
   */
  public void doLayout() {
    super.doLayout();
    int newSize = (int) screen.getMaxS() - 1;
    if (newSize != getBufferSize() && automaticBufferSize) {
      setBufferSize(newSize);
      ((TimedDataBufferView) dataView).adaptTimeLimits();
    }
  }
  
}
