package physbeans.math;

/**
 * Collection of special polynomials
 */
public class SpecialPolynomials {

  /**
   *
   *  Gets the value Hn(x) of the Hermite polynomial of order n.
   */
  static public double H(int n, double x) {
    double y = 0.0;
    double yp = 2.0 * x;
    double ypp = 1.0;

    if (n == 0) {
      return ypp;
    }

    if (n == 1) {
      return yp;
    }

    for (int i = 2; i <= n; i++) {
      y = 2.0 * x * yp - 2.0 * (i - 1) * ypp;
      ypp = yp;
      yp = y;
    }

    return y;
  }

  /**
   * Gets the value La_n^(m)(x) of the generalized Laguerre polynomial.
   */
  static public double La(int n, int m, double x) {
    // algorithm from http://en.wikipedia.org/wiki/Laguerre_polynomials
   
    double l0;
    double l1 = 0.0;
    double result = 1.0;
    for (int i = 1; i <= n; i++) {
      l0 = l1;
      l1 = result;
      result = ((2 * i - 1 + m - x) * l1 - (i - 1 + m) * l0) / i;
    }

    return result;
  }

  /**
   * Gets the coefficients of the Hermite polynomial of order n.
   */
  static public DVector Hcoeff(int n) {
    DVector coeff = new DVector(n + 1);
    DVector coeffP = new DVector(n + 1);
    DVector coeffPP = new DVector(n + 1);

    coeffPP.set(n, 1.0);
    if (n == 0) {
      return coeffPP;
    }

    coeffP.set(n - 1, 2.0);
    if (n == 1) {
      return coeffP;
    }

    for (int i = 2; i <= n; i++) {
      //y = 2.0 * x * yp - 2.0 * (i - 1) * ypp;

      DVector tt = coeffPP.times(2.0 * (i - 1));
      coeff = coeffP.shiftLeft().times(2.0).minus(tt);
      coeffPP = coeffP.copy();
      coeffP = coeff.copy();
    }

    return coeff;
  }
}
