package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;

/**
 * Function to measure the backforce of a linear mass-spring system to
 * the basement.
 *
 * Assumes that the first mass (index 0) is connected to the (moved) ground.
 */
public class BasementForce extends GenericNdFunction {

  protected LinearChainOscillator system;
  protected HarmonicBaseExcitation exciter;

  protected DVector computeFunction(DVector in) {
    if (system == null || exciter == null) {
      return null;
    }

    double c1 = system.getSpringStiffnesses().get(0);   // lowest spring
    double b1 = system.getDamperCoefficients().get(0);  // lowest damper

    int dim = in.getDimension() / 2;
    double x1 = in.get(0);
    double v1 = in.get(dim);
    double x0 = exciter.getBasementExcitation();
    double v0 = exciter.getBasementVelocity();

    return new DVector(c1*(x1 - x0) + b1*(v1 - v0));
  }

  /**
   * Gets the LinearChainOscillator system that is measured.
   */
  public LinearChainOscillator getSystem() {
    return system;
  }

  /**
   * Sets the LinearChainOscillator system that is measured.
   */
 public void setSystem(LinearChainOscillator s) {
    system = s;
    inform();
  }

 /**
   * Gets the HarmonicBaseExcitation.
   */
  public HarmonicBaseExcitation getExciter() {
    return exciter;
  }

  /**
   * Sets the HarmonicBaseExcitation.
   */
 public void setExciter(HarmonicBaseExcitation e) {
    exciter = e;
    inform();
  }
}
