package physbeans.phys;

import physbeans.model.*;
import physbeans.math.*;

/**
 * Grid2DScalarField with complex values and time evolution.
 */

// A clearer design would be to add time evolution as another hierarchy, since
// there are Grid2DScalarField's with and without it.
// On the other hand: I have (up to now) just 3 Grid2DScalarField altogether!
public abstract class ComplexGrid2DScalarField extends Grid2DScalarField
        implements Complex2DFunction, ComplexGridWithBoundary {

  protected double valI[][];    // attention: first index is y direction!

  /**
   * Initialises all grid structures and sets initial values.
   */
  protected void setupGrid() {
    super.setupGrid();

    if ((valI == null) || (valI[0].length != nx) || (valI.length != ny)) {
      valI = new double[ny][nx];
    }
  }

  /**
   * Gets the grid with the real parts of all computed scalar field values.
   */
  public double[][] getGridReal() {
    return val;
  }

  /**
   * Gets the grid with the imaginary parts of all computed scalar field values.
   */
  public double[][] getGridImag() {
    return valI;
  }

  /**
   * Gets the grid with the real parts.
   */
  public double[][] getGrid() {
    return getGridReal();
  }

  /**
   * Returns complex field value using bilinear interpolation of grid points.
   */
  public Complex getComplexValue(double x, double y) {
    double re = interp.getValue(x, y);

    // reuse interpolator
    interp.setGrid(valI);
    double im = interp.getValue(x, y);
    interp.setGrid(val);

    return new Complex(re, im);
  }

  /**
   * Returns the total norm of the complex scalar field defined by
   *    N = \int dx dy |psi(x, y)|^2
   * Values on the boundaries are discarded.
   */
  public double getTotalNorm() {
    double n = 0.0;

    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = bwY; j < ny - bwY; j++) {
        n += val[j][i] * val[j][i];
        n += valI[j][i] * valI[j][i];
      }
    }

    return n * getDX() * getDY();
  }

  public void restart() {
  }
}
