package physbeans.phys;

import static java.lang.Math.*;

/**
 * Initializes the grid values of a Grid2DScalarField.
 */
public abstract class ComplexGridInitializer {

  protected ComplexGrid2DScalarField field;

  /**
   * Sets initial values for the ScalarField f.
   */
  public abstract void setGrid();

  /**
   * Sets the field that is initialized.
   */
  public void setField(ComplexGrid2DScalarField f) {
    if (f != null) {
      field = f;
      setGrid();
    }
  }

  /**
   * Normalizes the scalar field.
   */
  protected void normalizeField() {
    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    int bwX = field.getBoundaryWidthX();
    int bwY = field.getBoundaryWidthY();

    double[][] gridR = field.getGridReal();
    double[][] gridI = field.getGridImag();

    double fac = 1.0 / sqrt(field.getTotalNorm());
    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = bwY; j < ny - bwY; j++) {
        gridR[j][i] = fac * gridR[j][i];
        gridI[j][i] = fac * gridI[j][i];
      }
    }
  }

  /**
   * Sets all values in the boundary to 0.
   */
  protected void clearBoundaries() {
    if (field == null) {
      return;
    }

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    int bwX = field.getBoundaryWidthX();
    int bwY = field.getBoundaryWidthY();

    double[][] gridR = field.getGridReal();
    double[][] gridI = field.getGridImag();

    for (int i = 0; i < bwX; i++) {
      for (int j = 0; j < ny; j++) {
        gridR[j][i] = 0.0;
        gridI[j][i] = 0.0;
      }
    }

    for (int i = nx - bwX; i < nx; i++) {
      for (int j = 0; j < ny; j++) {
        gridR[j][i] = 0.0;
        gridI[j][i] = 0.0;
      }
    }

    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = 0; j < bwY; j++) {
        gridR[j][i] = 0.0;
        gridI[j][i] = 0.0;
      }
    }

    for (int i = bwX; i < nx - bwX; i++) {
      for (int j = ny - bwY; j < ny; j++) {
        gridR[j][i] = 0.0;
        gridI[j][i] = 0.0;
      }
    }
  }

  /**
   * Forward values to the field.
   */
  protected void updateField() {
    if (field != null) {
      field.restart();
    }
  }
}
