package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.math.*;

/**
 * Shows a schematic diagram of an isex gas turbine consisting of 3 compressors,
 * 2 turbines and 5 heaters/coolers.<br>
 * The temperature of the gas in one of the parts (the "active" part) is
 * displayed with a color code. An additional input defines the number of this
 * part.
 */
public class IsexView extends TurbineView implements Serializable {
  
  public IsexView() {
    super();
  }
  
  /**
   * Redraws the complete turbine according to driving parameters.
   */
  public void paintLayer(Graphics2D g) {
    int phase = (int) Math.round(model.get(1));
    
    drawCompressorV(g, new DVector(0.0,0.6), 0.2, (phase%10 == 0));
    drawHeaterV(g, new DVector(0.0,1.4), 0.2, (phase%10 == 1));
    drawCompressorV(g, new DVector(0.0,2.2), 0.2, (phase%10 == 2));
    drawHeaterV(g, new DVector(0.0,3.0), 0.2, (phase%10 == 3));
    drawCompressorV(g, new DVector(0.0,3.8), 0.2, (phase%10 == 4));
    
    drawHeaterH(g, new DVector(2.0,4.4), 1.8, (phase%10 == 5));
    
    drawTurbineV(g, new DVector(4.0,3.55), 0.4, (phase%10 == 6));
    drawHeaterV(g, new DVector(4.0,2.2), 0.3, (phase%10 == 7));
    drawTurbineV(g, new DVector(4.0,0.85), 0.4, (phase%10 == 8));
    
    drawHeaterH(g, new DVector(2.0,0.0), 1.8, (phase%10 == 9));
    
    // dashed line between cooler and heater
    Stroke oldStroke = g.getStroke();
    Stroke dashed = new BasicStroke(0.02f, BasicStroke.CAP_BUTT,
        BasicStroke.JOIN_MITER, 10.0f, new float[]{0.05f}, 0.0f);
    Line2D.Double line = new Line2D.Double(2.0, 0.3, 2.0, 4.2);
    g.setStroke(dashed);
    g.draw(line);
    g.setStroke(oldStroke);
  }
  
}
